@@ -8,7 +8,15 @@ from binpatch.find import find
 
 class Find(Pattern):
     versions = {
+        '3.x': {
+            '3.1.3': '1357.5.30~6'
+        },
         '4.x': {
+            '4.0': '1504.50.73~2',
+            '4.0.1': '1504.50.73~2',
+            '4.0.2': '1504.50.80~1',
+            '4.1': '1504.55.33~10',
+            '4.2.1': '1504.58.28~3',
             '4.3': '1735.46~2',
             '4.3.1': '1735.46~2',
             '4.3.2': '1735.46~10',
@@ -104,6 +112,10 @@ class Find(Pattern):
         patterns = self.form_sandbox_profile()
         return self.findOffset(patterns)
 
+    def find_seatbelt_profile(self):
+        patterns = self.form_seatbelt_profile()
+        return self.findOffset(patterns)
+
     def getVersion(self):
         pattern = b'root:xnu'
         pattern_len = len(pattern)
@@ -133,52 +145,75 @@ class Find(Pattern):
             'nor_llb_3': False,
             'nor_llb_4': False,
             'nor_llb_5': False,
-            'sandbox_profile': False
+            'sandbox_profile': False,
+            'seatbelt_profile': False
         }
 
-        for base in self.versions:
-            versions = self.versions[base]
+        try:
+            for base in self.versions:
+                versions = self.versions[base]
 
-            for version in versions:
-                if versions[version] == version_string:
-                    self.version = version
+                for version in versions:
+                    if versions[version] == version_string:
+                        self.version = version
 
-                    if base == '4.x':
-                        if self.version in ('4.3', '4.3.1'):
-                            to_find['sandbox_profile'] = True
+                        if base == '3.x':
+                            to_find['nor_signature'] = True
+                            to_find['nor_llb_1'] = True
+                            to_find['nor_llb_2'] = True  # UNSURE
+                            to_find['nor_llb_3'] = True  # UNSURE
+                            to_find['nor_llb_4'] = True
+                            to_find['nor_llb_5'] = True
 
-                        to_find['debug_enabled'] = True
-                        to_find['vm_map_enter'] = True
-                        to_find['amfi_memcmp'] = True
-                        to_find['amfi_trust_cache'] = True
-                        to_find['nor_signature'] = True
-                        to_find['nor_llb_1'] = True
-                        to_find['nor_llb_2'] = True
-                        to_find['nor_llb_3'] = True
-                        to_find['nor_llb_4'] = True
-                        to_find['nor_llb_5'] = True
+                        elif base == '4.x':
+                            if self.version in ('4.3', '4.3.1'):
+                                to_find['sandbox_profile'] = True
 
-                    elif base == '5.x':
-                        to_find['debug_enabled'] = True,
-                        to_find['amfi_memcmp'] = True
-                        to_find['nor_llb_1'] = True
-                        to_find['nor_llb_2'] = True
-                        to_find['nor_llb_3'] = True
-                        to_find['nor_llb_4'] = True
-                        to_find['nor_llb_5'] = True
-                        to_find['nor_signature'] = True
+                            if self.version in ('4.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
+                                to_find['vm_map_enter'] = True
+
+                            if self.version == '4.0':
+                                to_find['seatbelt_profile'] = True
 
-                    elif base == '6.x':
-                        if version in ('6.1.3', '6.1.6'):
                             to_find['debug_enabled'] = True
+                            to_find['amfi_memcmp'] = True
                             to_find['amfi_trust_cache'] = True
-                            to_find['sandbox_mac_label_get'] = True
-                            to_find['sandbox_entitlement_container_required'] = True
+                            to_find['nor_signature'] = True
+                            to_find['nor_llb_1'] = True
+                            to_find['nor_llb_2'] = True
+                            to_find['nor_llb_3'] = True
+                            to_find['nor_llb_4'] = True
+                            to_find['nor_llb_5'] = True
 
-                        to_find['vm_map_enter'] = True
-                        to_find['tfp0'] = True
-                        to_find['nor_llb_1'] = True
-                        to_find['nor_llb_2'] = True
+                        elif base == '5.x':
+                            to_find['debug_enabled'] = True,
+                            to_find['amfi_memcmp'] = True
+                            to_find['nor_llb_1'] = True
+                            to_find['nor_llb_2'] = True
+                            to_find['nor_llb_3'] = True
+                            to_find['nor_llb_4'] = True
+                            to_find['nor_llb_5'] = True
+                            to_find['nor_signature'] = True
+
+                        elif base == '6.x':
+                            if version in ('6.1.3', '6.1.6'):
+                                to_find['debug_enabled'] = True
+                                to_find['amfi_trust_cache'] = True
+                                to_find['sandbox_mac_label_get'] = True
+                                to_find['sandbox_entitlement_container_required'] = True
+
+                            to_find['vm_map_enter'] = True
+                            to_find['tfp0'] = True
+                            to_find['nor_llb_1'] = True
+                            to_find['nor_llb_2'] = True
+
+                        raise StopIteration
+
+        # Make sure we are only patching once. Some versions have the same
+        # kernel version string.
+
+        except StopIteration:
+            pass
 
         for patch in to_find:
             func_names = dir(self)
