@@ -11,7 +11,13 @@ class Pattern(Instructions):
         return f'{instruction["mnemonic"]} {instruction["op_str"]}'
 
     def form_vm_map_enter(self):
-        if self.version == '4.3.3':
+        if self.version in ('4.0', '4.0.1', '4.0.2', '4.1'):
+            pattern = (
+                b'\x16\xf0\x02\x0f',
+                b'\x30\xd1'
+            )
+
+        elif self.version in ('4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\x18\xf0\x02\x0f',
                 b'\x2e\xd1'
@@ -36,7 +42,7 @@ class Pattern(Instructions):
         return pattern
 
     def form_debug_enabled(self):
-        if self.version == '4.3.3':
+        if self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\x00\x00\x00\x00',  # This is the value
                 b'\x01\x00\x00\x00',
@@ -59,7 +65,7 @@ class Pattern(Instructions):
         return pattern
 
     def form_amfi_trust_cache(self):
-        if self.version == '4.3.3':
+        if self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\x4f\xf0\xff\x30',
                 b'\x2c\xe0'
@@ -93,7 +99,14 @@ class Pattern(Instructions):
         return pattern
 
     def form_amfi_memcmp(self):
-        if self.version == '4.3.3':
+        if self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1'):
+            pattern = (
+                b'\x00\xb1',
+                b'\x00\x24',
+                b'\x20\x46'
+            )
+
+        elif self.version in ('4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\xb4\x42',
                 b'\xea\xd1',
@@ -109,7 +122,13 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_signature(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3'):
+            pattern = (
+                b'\xff\xf7\x29\xff',
+                b'\xf8\xb1'
+            )
+
+        elif self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\xff\xf7\x25\xff',
                 b'\xf8\xb1'
@@ -126,7 +145,13 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_llb_1(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3'):
+            pattern = (
+                b'\xff\xf7\x10\xff',
+                b'\x00\x38'
+            )
+
+        elif self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\xff\xf7\x0c\xff',
                 b'\x00\x38'
@@ -153,7 +178,14 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_llb_2(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3'):
+            pattern = (
+                b'\x26\x4b',
+                b'\x98\x47',
+                b'\x28\xb9'
+            )
+
+        elif self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\x02\x21',
                 b'\x7c\x4b',
@@ -180,7 +212,13 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_llb_3(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3'):
+            pattern = (
+                b'\xff\xf7\x07\xff',
+                b'\x08\xb9'
+            )
+
+        elif self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\xff\xf7\xab\xfd',
                 b'\x04\x46',
@@ -197,7 +235,13 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_llb_4(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3'):
+            pattern = (
+                b'\xff\xf7\x4e\xfe',
+                b'\x78\xb1'
+            )
+
+        elif self.version in ('4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\xff\xf7\x50\xfc',
                 b'\x00\xb3'
@@ -214,7 +258,7 @@ class Pattern(Instructions):
         return pattern
 
     def form_nor_llb_5(self):
-        if self.version == '4.3.3':
+        if self.version in ('3.1.3', '4.0', '4.0.1', '4.0.2', '4.1', '4.2.1', '4.3', '4.3.1', '4.3.2', '4.3.3'):
             pattern = (
                 b'\x4f\xf0\xff\x30',
                 b'\x2d\xe0'
@@ -228,3 +272,21 @@ class Pattern(Instructions):
             )
 
         return pattern
+
+    def form_sandbox_profile(self):
+        if self.version in ('4.3', '4.3.1'):
+            pattern = (
+                b'\x1b\x68',
+                b'\x13\xf0\x04\x0f',
+                b'\x04\xd0'
+            )
+
+        return pattern
+
+    def form_seatbelt_profile(self):
+        if self.version == '4.0':
+            pattern = (
+                b'seatbelt-profile',
+            )
+
+        return pattern
